@@ -6,20 +6,20 @@
 # Clean-up tmp and lock files
 #
 function cleanup() {
-	if [ "$ERROR" != "" ]; then
+	if [ -e $TMPDIR/error.log ]; then
 	        echo -e "         
 \n==============================================================================
 WARNING!        WARNING!        WARNING!        WARNING!        WARNING!
 ==============================================================================
-One or more errors occurred while slackpkg was running.                       
-One or more packages most likely could not be installed, or your mirror
-is having problems. It's a good idea recheck your mirror and run slackpkg
-again.
+One or more errors occurred while slackpkg was running:                       
+"
+		cat $TMPDIR/error.log
+		echo -e "
 =============================================================================="
 	fi    
 	echo
 	if [ "$DELALL" = "on" ] && [ "$NAMEPKG" != "" ]; then
-		rm $TEMP/$NAMEPKG &>/dev/null
+		rm $CACHEPATH/$NAMEPKG &>/dev/null
 	fi		
 	( rm -f /var/lock/slackpkg.$$ && rm -rf $TMPDIR ) &>/dev/null
 	exit
@@ -34,19 +34,19 @@ function system_checkup() {
 	if [ "$WORKDIR" = "" ]; then
 		echo -e "\
 \nYou need to upgrade your slackpkg.conf.\n\
-This is a new slackpkg version and many changes happens in config files.\n\
-In ${CONF}/slackpkg.conf.new there is a sample of the new configuration.\n\
-\nAfter update your configuration run: slackpkg update\n" 
+This is a new slackpkg version and many changes happened in config files.\n\
+In ${CONF}/slackpkg.conf.new, there is a sample of the new configuration.\n\
+\nAfter updating your configuration file, run: slackpkg update\n" 
 		cleanup
 	fi
 
 	# Checking if another instance of slackpkg is running
 	#
-	if [ "`ls /var/lock/slackpkg.* 2>/dev/null`" ] && \
+	if [ "$(ls /var/lock/slackpkg.* 2>/dev/null)" ] && \
 		[ "$CMD" != "search" ]; then
 		echo -e "\
-\nAnother instance of slackpkg is running. If this isn't true, you can remove\n\
-/var/lock/slackpkg.* files and run slackpkg again.\n"
+\nAnother instance of slackpkg is running. If this is not correct, you can\n\
+remove /var/lock/slackpkg.* files and run slackpkg again.\n"
 		cleanup
 	else        
 		ls /var/lock/slackpkg.* &>/dev/null || \
@@ -57,9 +57,10 @@ In ${CONF}/slackpkg.conf.new there is a sample of the new configuration.\n\
 	#
 	if [ "$TMPDIR" = "FAILED" ]; then
 		echo -e "\
-\nSome problem writing slackpkg's temporary dir in /tmp.\n\
-Probably you don't had permissions to wrote on /tmp or it's full.\n\
-Fix it and run slackpkg again.\n"
+\nA problem was encountered writing to slackpkg's temporary dir in /tmp.\n\
+Check to ensure you have permissions to write in /tmp and make sure the\n\
+filesystem is not out of free space.  Run slackpkg again after correcting\n\
+the problem.\n"
 		cleanup
 	fi
 
@@ -67,9 +68,9 @@ Fix it and run slackpkg again.\n"
 	#                                               
 	if ! [ -f ${WORKDIR}/pkglist ] && [ "$CMD" != "update" ]; then
 		echo -e "\
-\nHey! Probably this is the first time you are running slackpkg.\n\
-Before install|upgrade|reinstall anything, you need to uncomment\n\
-one mirror in ${CONF}/mirrors and run:\n\n\
+\nThis appears to be the first time you have run slackpkg.\n\
+Before you install|upgrade|reinstall anything, you need to uncomment\n\
+ONE mirror in ${CONF}/mirrors and run:\n\n\
 \t# slackpkg update\n\n\
 You can see more information about slackpkg functions in slackpkg manpage."
 		cleanup
@@ -80,71 +81,71 @@ You can see more information about slackpkg functions in slackpkg manpage."
 	#                                                         
 	if [ "$SOURCE" = "" ]; then
 		echo -e "\
-\nYou don't have any mirror selected in ${CONF}/mirrors\n\
-Please, edit that file and uncomment one mirror. Slackpkg\n\
+\nYou do not have any mirror selected in ${CONF}/mirrors\n\
+Please edit that file and uncomment ONE mirror.  Slackpkg\n\
 only works with ONE mirror selected.\n"
 		cleanup
 	else
-		COUNT=`echo $SOURCE | wc -w | tr -d " "`
+		COUNT=$(echo $SOURCE | wc -w | tr -d " ")
 		if [ "$COUNT" != "1" ]; then
 			echo -e "\n\
-Slackpkg only works with ONE mirror selected. Please, edit your\n\
-${CONF}/mirrors and comment one or more lines. Two or more\n\
-mirrors uncommented isn't valid syntax.\n"
+Slackpkg only works with ONE mirror selected.  Please edit your\n\
+${CONF}/mirrors and comment all but one line - two or more\n\
+mirrors uncommented is not valid syntax.\n"
 			cleanup
 		fi
 	fi
 
 	# It will check if the mirror selected are ftp.slackware.com
-	# if are "ftp.slackware.com" tell to the user to choice other
+	# if set to "ftp.slackware.com" tell the user to choose another
 	#
 	if echo ${SOURCE} | grep "^ftp://ftp.slackware.com" &>/dev/null ; then
 		echo -e "\n\
-Please, use one of the mirrors.\n\
-ftp.slackware.com should be preserved so the\n\
-mirrors can be kept up-to-date.\n"
+Please use one of the mirrors.\n\
+ftp.slackware.com should be reserved so that the\n\
+official mirrors can be kept up-to-date.\n"
 		cleanup
 	fi
 
-	# Command line syntax checking 
-	#
-	if [ "$#" = "0" ]; then
-		usage
-	fi
-
-	if [ "$#" = "1" ] && [ "$CMD" != "update" ] && \
-		[ "$CMD" != "upgrade-all" ] && [ "$CMD" != "install-new" ] && \
-		[ "$CMD" != "blacklist" ] && [ "$CMD" != "clean-system" ]; then
-		usage
-	fi
-	
-	# Checking if the user have the permissions to install/upgrade/update
+	# Checking if the user has the permissions to install/upgrade/update
 	#                                                                    
-	if [ "`id -u`" != "0" ] && [ "$CMD" != "search" ] && [ "$CMD" != "info" ]; then
+	if [ "$(id -u)" != "0" ] && \
+	   [ "$CMD" != "search" ] && \
+	   [ "$CMD" != "check-updates" ] && \
+	   [ "$CMD" != "info" ]; then
 		echo -e "\n\
-Only root can install, upgrade or remove packages.\n\
-Please, log as root or call your system administrator.\n"
+Only root can install, upgrade, or remove packages.\n\
+Please log in as root or contact your system administrator.\n"
 		cleanup
 	fi          
 
 	# Check if the mirror are local (cdrom or file)
 	#
-	MEDIA=`echo ${SOURCE} | cut -f1 -d:`
-	if [ "$MEDIA" = "cdrom" ] || [ "$MEDIA" = "file" ]; then
-		SOURCE=/`echo ${SOURCE} | cut -f3- -d/`
+	MEDIA=$(echo ${SOURCE} | cut -f1 -d:)
+	if [ "$MEDIA" = "cdrom" ] || [ "$MEDIA" = "file" ] || \
+	   [ "$MEDIA" = "local" ]; then
+		SOURCE=/$(echo ${SOURCE} | cut -f3- -d/)
 		LOCAL=1
 	fi
 
+	# Check if the "which" command is there
+	if ! which which 1>/dev/null 2>/dev/null ; then
+		echo -e "\n\
+No 'which' command found, please install it if you want to\n\
+use slackpkg.\n"
+		cleanup
+	fi
+
 	# Check if we have md5sum in the PATH. Without md5sum, disables
 	# md5sum checks
 	#
-	if ! [ `which md5sum 2>/dev/null` ]; then
+	if ! [ $(which md5sum 2>/dev/null) ]; then
 		CHECKPKG=off
 	elif ! [ -f ${WORKDIR}/CHECKSUMS.md5 ] && \
 		[ "$CMD" != "update" ] && \
 		[ "$CHECKPKG" = "on" ]; then
 		echo -e "\n\
-No CHECKSUMS.md5 found! Please, disable md5sums checking\n\
+No CHECKSUMS.md5 found!  Please disable md5sums checking\n\
 on your ${CONF}/slackpkg.conf or run slackpkg update\n\
 to download a new CHECKSUMS.md5 file.\n"
 		cleanup
@@ -154,8 +155,8 @@ to download a new CHECKSUMS.md5 file.\n"
 	#
 	if ! [ "$(which awk 2>/dev/null)" ]; then
 		echo -e "\n\
-No awk found! Please, install awk before you run slackpkg.\n\
-Without awk, slackpkg doesn't work in any way.\n"
+awk package not found! Please install awk before you run slackpkg,\n\
+as slackpkg cannot function without awk.\n"
 		cleanup
 	fi
 
@@ -164,35 +165,51 @@ Without awk, slackpkg doesn't work in any way.\n"
 	if ! [ "$(which gpg 2>/dev/null)" ] && [ "${CHECKGPG}" = "on" ]; then
 		CHECKGPG=off
 		echo -e "\n\
-No gpg found!!! Please, disable GPG in ${CONF}/slackpkg.conf or install\n\
+gpg package not found!  Please disable GPG in ${CONF}/slackpkg.conf or install\n\
 the gnupg package.\n\n\
-To disable GPG, edit slackpkg.conf and add one line with CHECKGPG=off.\n\
-You can see an example in slackpkg.conf.new.\n "
+To disable GPG, edit slackpkg.conf and change the value of the CHECKGPG variable\n\
+to "off" - you can see an example in the original slackpkg.conf.new file distributed\n\
+with slackpkg.\n"
 		sleep 5
 	fi 
 
 	# Check if the Slackware GPG key are found in the system
 	#                                                       
-	GPGFIRSTTIME="`gpg --list-keys \"$SLACKKEY\" 2>/dev/null \
-			| grep -c \"$SLACKKEY\"`"
-	if [ "$GPGFIRSTTIME" = "0" ] && [ "$CMD" != "search" ] && [ "$CMD" != "info" ] && \
-			[ "$CMD" != "update" ] && [ "$CHECKGPG" = "on" ]; then
+	GPGFIRSTTIME="$(gpg --list-keys \"$SLACKKEY\" 2>/dev/null \
+			| grep -c "$SLACKKEY")"
+	if [ "$GPGFIRSTTIME" = "0" ] && \
+		[ "$CMD" != "search" ] && \
+		[ "$CMD" != "info" ] && \
+		[ "$CMD" != "update" ] && \
+		[ "$CMD" != "check-updates" ] && \
+		[ "$CHECKGPG" = "on" ]; then
 		echo -e "\n\
 You need the GPG key of $SLACKKEY.\n\
 To download and install that key, run:\n\n\
 \t# slackpkg update gpg\n\n\
-You can disable GPG checking, too. But it isn't a good idea.\n\
-To disable GPG, edit slackpkg.conf and add one line with CHECKGPG=off.\n\
-You can see an example in slackpkg.conf.new.\n"
+You can disable GPG checking too, but it is not a good idea.\n\
+To disable GPG, edit slackpkg.conf and change the value of the CHECKGPG variable\n\
+to "off" - you can see an example in the original slackpkg.conf.new file distributed\n\
+with slackpkg.\n"
 		cleanup
 	fi
+
+	if [ "$BATCH" = "on" ] || [ "$BATCH" = "ON" ]; then
+		DIALOG=off
+		MORECMD=cat
+		if [ "$DEFAULT_ANSWER" = "" ]; then
+			DEFAULT_ANSWER=n
+		fi
+	else
+		MORECMD=more
+	fi
 	echo 
 }
 
 # Got the name of a package, without version-arch-release data
 #
 function cutpkg() {
-	echo ${1/.tgz/} | awk -F- 'OFS="-" { 
+	echo ${1/%.t[blxg]z/} | awk -F- 'OFS="-" { 
 				if ( NF > 3 ) { 
 					NF=NF-3
 					print $0 
@@ -209,33 +226,38 @@ function usage() {
 slackpkg - version $VERSION\n\
 \nUsage: \tslackpkg update [gpg]\t\tdownload and update files and 
 \t\t\t\t\tpackage indexes
+\tslackpkg check-updates\t\tcheck if there is any news on
+\t\t\t\t\tSlackware's ChangeLog.txt
 \tslackpkg install package\tdownload and install packages 
 \tslackpkg upgrade package\tdownload and upgrade packages
 \tslackpkg reinstall package\tsame as install, but for packages 
 \t\t\t\t\talready installed
 \tslackpkg remove package\t\tremove installed packages
 \tslackpkg clean-system\t\tremove all packages which are not 
-\t\t\t\t\tpresent in slackware distribution.
-\t\t\t\t\tGood to keep the house in order
+\t\t\t\t\tpresent in the official Slackware 
+\t\t\t\t\tpackage set. Good to keep the house
+\t\t\t\t\tin order
 \tslackpkg upgrade-all\t\tsync all packages installed in your 
 \t\t\t\t\tmachine with the selected mirror. This
-\t\t\t\t\tis the "true" way to upgrade all your 
-\t\t\t\t\tmachine.
-\tslackpkg install-new\t\tinstall packages which are added in
-\t\t\t\t\tslackware distribution. 
+\t\t\t\t\tis the "correct" way to upgrade all of 
+\t\t\t\t\tyour machine.
+\tslackpkg install-new\t\tinstall packages which are added to
+\t\t\t\t\tthe official Slackware package set.
 \t\t\t\t\tRun this if you are upgrading to another
-\t\t\t\t\tslackware version or using "current".
+\t\t\t\t\tSlackware version or using "current".
 \tslackpkg blacklist\t\tBlacklist a package. Blacklisted
-\t\t\t\t\tpackages cannot be upgraded, installed
+\t\t\t\t\tpackages cannot be upgraded, installed,
 \t\t\t\t\tor reinstalled by slackpkg
-\tslackpkg download\t\tJust download (do not install) a package
+\tslackpkg download\t\tOnly download (do not install) a package
 \tslackpkg info package\t\tShow package information 
 \t\t\t\t\t(works with only ONE package)
 \tslackpkg search file\t\tSearch for a specific file in the
 \t\t\t\t\tentire package collection
+\tslackpkg new-config\t\tSearch for new configuration files and
+\t\t\t\t\task to user what to do with them.
 \nYou can see more information about slackpkg usage and some examples
 in slackpkg's manpage. You can use partial package names (such as x11
-instead x11-devel, x11-docs, etc), or even slackware series
+instead x11-devel, x11-docs, etc), or even Slackware series
 (such as "n","ap","xap",etc) when searching for packages.
 "
 	cleanup
@@ -243,12 +265,13 @@ instead x11-devel, x11-docs, etc), or even slackware series
 
 # Verify if the package was corrupted by checking md5sum
 #
-function checkpkg() {
+function checkmd5() {
 	local MD5ORIGINAL
 	local MD5DOWNLOAD
 
-	MD5ORIGINAL=`grep "${NAMEPKG}$" ${WORKDIR}/CHECKSUMS.md5| cut -f1 -d \ `
-	MD5DOWNLOAD=`md5sum ${TEMP}/${1} | cut -f1 -d \ `
+	MD5ORIGINAL=$(	grep -v "/source/" ${CHECKSUMSFILE} |\
+			grep -m1 "/$(basename $1)$" | cut -f1 -d \ )
+	MD5DOWNLOAD=$(md5sum ${1} | cut -f1 -d \ )
 	if [ "$MD5ORIGINAL" = "$MD5DOWNLOAD" ]; then
 		echo 1 
 	else
@@ -257,7 +280,7 @@ function checkpkg() {
 }
 
 function checkgpg() {
-	gpg --verify ${TEMP}/${1}.asc ${TEMP}/${1} 2>/dev/null && echo "1" || echo "0"
+	gpg --verify ${1}.asc ${1} 2>/dev/null && echo "1" || echo "0"
 }
 
 
@@ -289,12 +312,12 @@ function givepriority {
 	
         for DIR in $FIRST $SECOND $THIRD $FOURTH $FIFTH ; do
 		[ "$PKGDATA" ] && break
-                PKGDATA=( `grep "^${DIR} ${ARGUMENT} " ${WORKDIR}/pkglist` )
+                PKGDATA=( $(grep "^${DIR} ${ARGUMENT} " ${WORKDIR}/pkglist) )
                 if [ "$PKGDATA" ]; then
                         checkblacklist
                         if [ "$?" = "1" ]; then
 				NAME=${PKGDATA[1]}
-                                FULLNAME=`echo "${PKGDATA[5]}"`
+                                FULLNAME=$(echo "${PKGDATA[5]}.${PKGDATA[7]}")
 			else
 				unset PKGDATA
 				unset FULLNAME
@@ -318,47 +341,47 @@ function makelist() {
 
 	case "$CMD" in
 		clean-system)
-			echo -n "Looking for packages to remove. Please, wait... "
+			echo -n "Looking for packages to remove. Please wait... "
 		;;
 		upgrade-all)
-			echo -n "Looking for packages to upgrade. Please, wait... "
+			echo -n "Looking for packages to upgrade. Please wait... "
 		;;
 		install-new)
-			echo -n "Looking for NEW packages to install. Please, wait... "
+			echo -n "Looking for NEW packages to install. Please wait... "
 		;;
 		*)
-			echo -n "Looking for $(echo $INPUTLIST | tr -d '\\') in package list. Please, wait... "
+			echo -n "Looking for $(echo $INPUTLIST | tr -d '\\') in package list. Please wait... "
 		;;
 	esac
 
 	case "$CMD" in
 		download)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u`; do
-					LIST="$LIST `grep " ${i} " ${WORKDIR}/pkglist | cut -f6 -d \ `"
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u); do
+					LIST="$LIST $(grep " ${i} " ${WORKDIR}/pkglist | cut -f6,8 -d\  --output-delimiter=.)"
 				done
-				LIST="`echo -e $LIST | sort -u`"
+				LIST="$(echo -e $LIST | sort -u)"
 			done
 		;;
 		blacklist)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
-						grep -w -- "${ARGUMENT}" | cut -f2 -d\  | sort -u`; do
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
+						grep -w -- "${ARGUMENT}" | cut -f2 -d\  | sort -u); do
 					grep -qx "${i}" ${CONF}/blacklist || LIST="$LIST $i"
 				done
 			done
 		;;
 		install|upgrade|reinstall)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u`; do
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u); do
 					givepriority $i
 					[ ! "$FULLNAME" ] && continue
 
 					case $CMD in
 						'upgrade')
-							VRFY=`cut -f6 -d\  ${TMPDIR}/tmplist | \
-							      grep -x "${NAME}-[^-]\+-\(noarch\|${ARCH}\)-[^-]\+"`
-							[ "${FULLNAME}" != "${VRFY}" ]  && \
+							VRFY=$(cut -f6 -d\  ${TMPDIR}/tmplist | \
+							      grep -x "${NAME}-[^-]\+-\(noarch\|fw\|${ARCH}\)-[^-]\+")
+							[ "${FULLNAME/%.t[blxg]z/}" != "${VRFY}" ]  && \
 										[ "${VRFY}" ] && \
 								LIST="$LIST ${FULLNAME}"
 						;;
@@ -367,7 +390,7 @@ function makelist() {
 								LIST="$LIST ${FULLNAME}"
 						;;
 						'reinstall')
-							grep -q " ${FULLNAME} " ${TMPDIR}/tmplist && \
+							grep -q " ${FULLNAME/%.t[blxg]z} " ${TMPDIR}/tmplist && \
 								LIST="$LIST ${FULLNAME}"
 						;;
 					esac
@@ -375,10 +398,10 @@ function makelist() {
 			done
 		;;
 		remove)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
-					  	grep -w -- "${ARGUMENT}" | cut -f6 -d\  | sort -u`; do
-					PKGDATA=( `grep -w -- "$i" ${TMPDIR}/tmplist` )
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
+					  	grep -w -- "${ARGUMENT}" | cut -f6 -d\  | sort -u); do
+					PKGDATA=( $(grep -w -- "$i" ${TMPDIR}/tmplist) )
 					[ ! "$PKGDATA" ] && continue
 					checkblacklist
 					LIST="$LIST ${PKGDATA[5]}" 
@@ -387,29 +410,31 @@ function makelist() {
 			done
 		;;
 		clean-system)
-	                for i in `cut -f6 -d\  ${TMPDIR}/tmplist`; do
-				NAME=`cutpkg $i`
-				if [ `cut -f2 -d\  ${WORKDIR}/pkglist |grep -cx "${NAME}"` = "0" ] &&
-					[ `grep -cx "${NAME}" $CONF/blacklist` = 0 ]; then
+	                for i in $(cut -f6 -d\  ${TMPDIR}/tmplist); do
+				NAME=$(cutpkg $i)
+				if [ $(cut -f2 -d\  ${WORKDIR}/pkglist |grep -cx "${NAME}") = "0" ] &&
+					[ $(grep -cx "${NAME}" $CONF/blacklist) = 0 ]; then
 					LIST="$LIST $i"
                         	fi
                 	done
 		;;
 		upgrade-all)
-			for i in `cut -f2 -d\  ${TMPDIR}/tmplist`; do
+			for i in $(cut -f2 -d\  ${TMPDIR}/tmplist); do
 
 				givepriority ${i}
 				[ ! "$FULLNAME" ] && continue
 
-				VRFY=`cut -f6 -d\  ${TMPDIR}/tmplist | grep -x "${NAME}-[^-]\+-\(noarch\|${ARCH}\)-[^-]\+"`
-				[ "${FULLNAME}" != "${VRFY}" ]  && \
+				VRFY=$(cut -f6 -d\  ${TMPDIR}/tmplist | grep -x "${NAME}-[^-]\+-\(noarch\|fw\|${ARCH}\)-[^-]\+")
+				[ "${FULLNAME/%.t[blxg]z}" != "${VRFY}" ]  && \
 							[ "${VRFY}" ] && \
 					LIST="$LIST ${FULLNAME}"
 			done
 		;;
 		install-new)
-			for i in `awk -f /usr/libexec/slackpkg/install-new.awk ${WORKDIR}/ChangeLog.txt |\
-				  sort -u ` dialog aaa_terminfo fontconfig ; do
+			for i in $(awk -f /usr/libexec/slackpkg/install-new.awk ${WORKDIR}/ChangeLog.txt |\
+				  sort -u ) dialog aaa_terminfo fontconfig \
+				ntfs-3g ghostscript wqy-zenhei-font-ttf \
+				xbacklight xf86-video-geode ; do
 	
 				givepriority $i
 				[ ! "$FULLNAME" ] && continue
@@ -419,17 +444,26 @@ function makelist() {
 			done
 		;;
 	esac
-	LIST=`echo -e $LIST | tr \  "\n"`
+	LIST=$(echo -e $LIST | tr \  "\n" | uniq )
 	echo -e "DONE\n"
 }
 
 # Function to count total of packages
 #
 function countpkg() {
-	local COUNTPKG=`echo -e "$1" | wc -w`
+	local COUNTPKG=$(echo -e "$1" | wc -w)
 
 	if [ "$COUNTPKG" != "0" ]; then
-		echo -e "Total of package(s): $COUNTPKG\n"
+		echo -e "Total package(s): $COUNTPKG\n"
+	fi
+}
+
+function answer() {
+	if [ "$BATCH" = "on" ]; then
+		ANSWER="$DEFAULT_ANSWER"
+		echo $DEFAULT_ANSWER
+	else
+		read ANSWER
 	fi
 }
 
@@ -440,10 +474,11 @@ function showlist() {
 	local ANSWER
 	local i
 
-	for i in $1; do echo $i; done | more 
+	for i in $1; do echo $i; done | $MORECMD
 	echo
 	countpkg "$1"
-	echo -e "Do you wish to $2 selected packages (Y/n)? \c"; read ANSWER
+	echo -e "Do you wish to $2 selected packages (Y/n)? \c"
+	answer
 	if [ "$ANSWER" = "N" -o "$ANSWER" = "n" ]; then
 		cleanup
 	else
@@ -466,66 +501,96 @@ function getfile() {
 #
 function getpkg() {
 	local ISOK="1"
-	local PKGNAME=( `grep -w -m 1 -- "$1" ${WORKDIR}/pkglist` )
+	local ERROR=""
+	local PKGNAME
 	local FULLPATH
+	local NAMEPKG
+	local CACHEPATH
 
-	NAMEPKG=${PKGNAME[5]}
+	PKGNAME=( $(grep -w -m 1 -- "${1/%.t[blxg]z/}" ${WORKDIR}/pkglist) )
+	NAMEPKG=${PKGNAME[5]}.${PKGNAME[7]}
 	FULLPATH=${PKGNAME[6]}
+	CACHEPATH=${TEMP}/${FULLPATH}
 
-	if [ "`ls ${TEMP}/$1 2>/dev/null`" = "" ]; then
+	# Create destination dir if it isn't there
+	if ! [ -d $CACHEPATH ]; then
+		mkdir -p $CACHEPATH
+	fi
+
+	if ! [ -e ${CACHEPATH}/${NAMEPKG} ]; then
 		echo -e "\nPackage: $1"
 		# Check if the mirror are local, if is local, copy files 
-		# to TEMP else, download packages from remote host and 
-		# put then in TEMP
+		# to CACHEPATH else, download packages from remote host and 
+		# put then in CACHEPATH
 		#
 		if [ "${LOCAL}" = "1" ]; then 
-                	echo -e "\tCopying $1..."
-			cp ${SOURCE}${FULLPATH}/${NAMEPKG} ${TEMP}
+                	echo -e "\tCopying $NAMEPKG..."
+			cp ${SOURCE}${FULLPATH}/${NAMEPKG} ${CACHEPATH}
 			if [ "$CHECKGPG" = "on" ]; then
-				cp ${SOURCE}${FULLPATH}/${NAMEPKG}.asc ${TEMP}
+				cp ${SOURCE}${FULLPATH}/${NAMEPKG}.asc ${CACHEPATH}
 			fi
 		else
-                	echo -e "\tDownloading $1..."
-			wget ${WGETFLAGS} -P ${TEMP} -nd ${SOURCE}${FULLPATH}/${NAMEPKG}
+                	echo -e "\tDownloading $NAMEPKG..."
+			wget ${WGETFLAGS} -P ${CACHEPATH} -nd ${SOURCE}${FULLPATH}/${NAMEPKG}
 			if [ "$CHECKGPG" = "on" ]; then
-				wget ${WGETFLAGS} -P ${TEMP} -nd ${SOURCE}${FULLPATH}/${NAMEPKG}.asc
+				wget ${WGETFLAGS} -P ${CACHEPATH} -nd ${SOURCE}${FULLPATH}/${NAMEPKG}.asc
 			fi
 		fi
+
+		if ! [ -e $CACHEPATH/$1 ]; then
+			ERROR="Not found"
+			ISOK="0"
+			echo -e "${NAMEPKG}:\t$ERROR" >> $TMPDIR/error.log
+		fi
 	else
-		echo -e "\tPackage $1 is already in cache, not downloading" 
+		echo -e "\tPackage $1 is already in cache - not downloading" 
 	fi
 
 	# If MD5SUM checks are enabled in slackpkg.conf, check the
 	# packages md5sum to detect if they are corrupt or not
 	#
-	if [ "$CHECKPKG" = "on" ]; then
-		ISOK=`checkpkg $1`
-		[ "$ISOK" = "0" ] && ERROR="md5sum"
+	if [ "$CHECKPKG" = "on" ] && [ "$ISOK" = "1" ]; then
+		ISOK=$(checkmd5 ${CACHEPATH}/$1)
+		if [ "$ISOK" = "0" ]; then 
+			ERROR="md5sum"
+			echo -e "${NAMEPKG}:\t$ERROR" >> $TMPDIR/error.log
+		fi
+		if [ "$CHECKGPG" = "on" ] && [ "$ISOK" = "1" ]; then
+			ISOK=$(checkmd5 ${CACHEPATH}/$1.asc)
+			if [ "$ISOK" = "0" ]; then 
+				ERROR="md5sum"
+				echo -e "${NAMEPKG}.asc:\t$ERROR" >> \
+							$TMPDIR/error.log
+			fi
+		fi
 	fi
 
 	# Check the package against its .asc. If you don't like this
 	# disable GPG checking in /etc/slackpkg/slackpkg.conf
 	#
-	if [ "$CHECKGPG" = "on" ]; then
-		ISOK=`checkgpg $1`
-		[ "$ISOK" = "0" ] && ERROR="gpg"
+	if [ "$CHECKGPG" = "on" ] && [ "$ISOK" = "1" ]; then
+		ISOK=$(checkgpg ${CACHEPATH}/$1)
+		if [ "$ISOK" = "0" ]; then 
+			ERROR="gpg"
+			echo -e "${NAMEPKG}:\t$ERROR" >> $TMPDIR/error.log
+		fi
 	fi
 
 	if [ "$ISOK" = "1" ]; then
 		case $2 in
 			installpkg)
-				echo -e "\tInstalling ${1/.tgz/}..."
+				echo -e "\tInstalling ${1/%.t[blxg]z/}..."
 			;;
 			upgradepkg)
-				echo -e "\tUpgrading ${1/.tgz/}..."
+				echo -e "\tUpgrading ${1/%.t[blxg]z/}..."
 			;;
 			*)
 				echo -e "\c"
 			;;
 		esac	
-		( cd $TEMP && $2 $1 ) 
-	else
-		rm $TEMP/$1 2>/dev/null
+		( cd $CACHEPATH && $2 $1 )
+	else 
+		rm $CACHEPATH/$1 2>/dev/null
 		echo -e "\tERROR - Package not installed! $ERROR error!" 
 	fi
 
@@ -533,13 +598,13 @@ function getpkg() {
 	# after installed/upgraded/reinstalled
 	#
 	if [ "$DELALL" = "on" ]; then
-		rm $TEMP/$1 $TEMP/${1}.asc 2>/dev/null
+		rm $CACHEPATH/$1 $CACHEPATH/${1}.asc 2>/dev/null
 	fi		
 }
 
 # Main logic to download and format package list, md5 etc.
 #
-function updatefilelists()
+function checkchangelog()
 {
 	if ! [ -e ${WORKDIR}/ChangeLog.txt ]; then
 		touch ${WORKDIR}/ChangeLog.txt
@@ -551,11 +616,27 @@ function updatefilelists()
 	# or different from our already existent ChangeLog.txt 
 	#
 	getfile ChangeLog.txt $TMPDIR/ChangeLog.txt
+	if ! grep -q "[a-z]" $TMPDIR/ChangeLog.txt ; then
+		echo -e "\
+\nError downloading from $SOURCE.\n\
+Please, check your mirror and try again."
+		cleanup
+	fi
+
 	if diff --brief ${WORKDIR}/ChangeLog.txt $TMPDIR/ChangeLog.txt ; then
+		return 0
+	else
+		return 1
+	fi
+}
+
+function updatefilelists()
+{
+	if checkchangelog ; then
 		echo -e "\
 \n\t\tNo changes in ChangeLog.txt between your last update and now.\n\
 \t\tDo you really want to download all other files (y/N)? \c"
-		read ANSWER
+		answer
 		if [ "$ANSWER" != "Y" ] && [ "$ANSWER" != "y" ]; then
 			cleanup
 		fi
@@ -575,14 +656,88 @@ function updatefilelists()
 			DIRS="$DIRS $i"
 	done
 
+	ISOK="1"
+	echo -e "\t\tChecksums"
+	getfile CHECKSUMS.md5 ${TMPDIR}/CHECKSUMS.md5
+	getfile CHECKSUMS.md5.asc ${TMPDIR}/CHECKSUMS.md5.asc
+	if ! [ -e "${TMPDIR}/CHECKSUMS.md5" ]; then
+		echo -e "\
+\n\t\tWARNING: Your mirror appears incomplete and is missing the\n\
+\t\t         CHECKSUMS.md5 file. We recommend you change your mirror\n\
+\t\t         so that package integrity can be verified against \n\
+\t\t         CHECKSUMS.md5.\n"
+		sleep 10
+	else
+		if [ "$CHECKGPG" = "on" ]; then
+			ISOK=$(checkgpg ${TMPDIR}/CHECKSUMS.md5)
+			if [ "$ISOK" = "0" ]; then 
+				rm $TMPDIR/CHECKSUMS.md5
+				rm $TMPDIR/CHECKSUMS.md5.asc
+				echo -e "\
+\n\t\tERROR: Verification of the  gpg signature on CHECKSUMS.md5\n\
+\t\t       failed! This could mean that the file is out of date\n\
+\t\t       or has been tampered with.\n"
+				cleanup
+			fi
+		else
+			echo -e "\
+\n\t\tWARNING: Without CHECKGPG, we can't check if this file is\n\
+\t\t         signed by:\n\
+\n\t\t         $SLACKKEY.\n\
+\n\t\t         Enabling CHECKGPG is highly recommended for best\n\
+\t\t         security.\n"
+				sleep 10
+		fi
+	fi
+
+	ISOK="1"
 	echo -e "\t\tPackage List"
 	getfile FILELIST.TXT $TMPDIR/FILELIST.TXT
-
 	if [ "$CHECKPKG" = "on" ]; then
-		echo -e "\t\tChecksums"
-		getfile CHECKSUMS.md5 ${WORKDIR}/CHECKSUMS.md5
+		CHECKSUMSFILE=$TMPDIR/CHECKSUMS.md5
+		ISOK=$(checkmd5 $TMPDIR/FILELIST.TXT)
 	fi
-		
+	if [ "$ISOK" = "1" ]; then 
+		if ! [ -e $WORKDIR/LASTUPDATE ]; then
+			echo "742868196" > $WORKDIR/LASTUPDATE
+		fi
+		LASTUPDATE=$(cat $WORKDIR/LASTUPDATE)
+		ACTUALDATE=$(date -d "$(head -1 $TMPDIR/FILELIST.TXT)" "+%s")
+		if [ $ACTUALDATE -lt $LASTUPDATE ]; then
+			echo -e "\
+\n\t\tFILELIST.TXT seems to be older than the last one.\n\
+\t\tDo you really want to continue (y/N)? \c"
+			answer
+			if [ "$ANSWER" != "Y" ] && [ "$ANSWER" != "y" ]; then
+				cleanup
+			fi
+			echo
+		fi
+		echo $ACTUALDATE > $WORKDIR/LASTUPDATE
+	else
+		rm $TMPDIR/FILELIST.TXT
+	fi
+	
+	if [ -e $TMPDIR/CHECKSUMS.md5 ]; then
+		FILELIST="$TMPDIR/CHECKSUMS.md5"
+	elif [ -e $TMPDIR/FILELIST.TXT ]; then
+		if [ "$ISOK" = "0" ]; then
+			echo -e "\
+\n\t\tERROR: CHECKSUMS.md5 signature doesn't match!\n\
+\t\t       We strongly recommend that you change your mirror\n\
+\t\t       to prevent security problems.\n"
+			cleanup
+		fi
+		sleep 10
+	  	FILELIST="$TMPDIR/FILELIST.TXT"
+	else
+		echo -e "\
+\n\t\tERROR: No CHECKSUMS.md5 and no FILELIST.TXT.\n\
+\t\t       We strongly recommend that you change your mirror\n\
+\t\t       to prevent security problems.\n"
+		cleanup
+	fi 
+
 	# Download all PACKAGES.TXT files
 	# 
 	echo -e "\t\tPackage descriptions"
@@ -593,9 +748,19 @@ function updatefilelists()
 	# Format FILELIST.TXT
 	#
 	echo -e "\tFormatting lists to slackpkg style..."
-	echo -e "\t\tPackage List"
-	grep "\.tgz" $TMPDIR/FILELIST.TXT| awk -f /usr/libexec/slackpkg/pkglist.awk > ${WORKDIR}/pkglist 
-		
+	echo -e "\t\tPackage List: using $( basename $FILELIST ) as source"
+	grep "\.t[blxg]z$" $FILELIST| \
+		awk -f /usr/libexec/slackpkg/pkglist.awk |\
+		sed -e 's/^M//g' > ${TMPDIR}/pkglist
+	cp ${TMPDIR}/pkglist ${WORKDIR}/pkglist		
+
+	# Create the slackware tree under TEMP directory
+	for i in $( cut -f7 -d\  ${WORKDIR}/pkglist | sort -u ) ; do
+	  if ! [ -d ${TEMP}/${i} ]; then
+	    mkdir -p ${TEMP}/${i}
+	  fi
+	done
+
 	# Format MANIFEST
 	#
 		
@@ -604,8 +769,9 @@ function updatefilelists()
 	MANFILES=""
 	for i in $DIRS; do
 		bunzip2 -c $TMPDIR/${i}-MANIFEST.bz2 | awk -f /usr/libexec/slackpkg/filelist.awk | \
-			gzip > ${WORKDIR}/${i}-filelist.gz
+			gzip > ${TMPDIR}/${i}-filelist.gz
 	done
+	cp ${TMPDIR}/*-filelist.gz ${WORKDIR}/
 
 	if [ -r ${WORKDIR}/filelist.gz ]; then
 		rm ${WORKDIR}/filelist.gz
@@ -619,6 +785,15 @@ function updatefilelists()
 		cat $TMPDIR/${i}-PACKAGES.TXT >> $TMPDIR/PACKAGES.TXT
 	done
 	cp $TMPDIR/PACKAGES.TXT ${WORKDIR}/PACKAGES.TXT
+
+	if [ -e $TMPDIR/CHECKSUMS.md5 ]; then
+		cp $TMPDIR/CHECKSUMS.md5 $WORKDIR/CHECKSUMS.md5 2>/dev/null
+	fi
+
+	if [ -e $TMPDIR/CHECKSUMS.md5.asc ]; then
+		cp $TMPDIR/CHECKSUMS.md5.asc \
+			$WORKDIR/CHECKSUMS.md5.asc 2>/dev/null
+	fi
 }
 
 function sanity_check() {
@@ -628,17 +803,17 @@ function sanity_check() {
 	local DOUBLEFILES
 	local ANSWER
 
-	for i in `ls -1 /var/log/packages | \
-		egrep -- "^.*-(${ARCH}|noarch)-[^-]+-upgraded"`; do
-		REVNAME=`echo ${i} | awk -F'-upgraded' '{ print $1 }'`
+	for i in $(ls -1 /var/log/packages | \
+		egrep -- "^.*-(${ARCH}|fw|noarch)-[^-]+-upgraded"); do
+		REVNAME=$(echo ${i} | awk -F'-upgraded' '{ print $1 }')
 		mv /var/log/packages/${i} /var/log/packages/${REVNAME}
 		mv /var/log/scripts/${i} /var/log/scripts/${REVNAME}
 	done 
-	for i in `ls -1 /var/log/packages | egrep "^.*-(${ARCH}|noarch)-[^-]+$"`; do
+	for i in $(ls -1 /var/log/packages | egrep "^.*-(${ARCH}|fw|noarch)-[^-]+$"); do
 		cutpkg $i
 	done | sort > $TMPDIR/list1
 	cat $TMPDIR/list1 | uniq > $TMPDIR/list2
-	FILES="`diff $TMPDIR/list1 $TMPDIR/list2 | grep '<' | cut -f2 -d\ `"
+	FILES="$(diff $TMPDIR/list1 $TMPDIR/list2 | grep '<' | cut -f2 -d\ )"
 	if [ "$FILES" != "" ]; then
 		for i in $FILES ; do
 			grep -qx "${i}" ${CONF}/blacklist && continue
@@ -648,17 +823,17 @@ function sanity_check() {
 	fi
 	if [ "$DOUBLEFILES" != "" ]; then
 		echo -e "\
-You have a broked /var/log/packages, with two versions of the same package.\n\
-The list of packages duplicated in your machine are show below, but don't\n\
-worry about this list, when you select your action, slackpkg will show a\n\
+You have a broken /var/log/packages - with two versions of the same package.\n\
+The list of packages duplicated in your machine are shown below, but don't\n\
+worry about this list - when you select your action, slackpkg will show a\n\
 better list:\n"
 		for i in $DOUBLEFILES ; do
 			ls -1 /var/log/packages |\
-				egrep -i -- "^${i}-[^-]+-(${ARCH}|noarch)-"
+				egrep -i -- "^${i}-[^-]+-(${ARCH}|fw|noarch)-"
 		done
 		echo -ne "\n\
-You can (B)lacklist or (R)emove these packages.\n\
-Select your action (B/R): "
+You can (B)lacklist, (R)emove, or (I)gnore these packages.\n\
+Select your action (B/R/I): "
 		read ANSWER
 		echo
 		case "$ANSWER" in
@@ -668,8 +843,8 @@ Select your action (B/R): "
 			;;
 			R|r)
 				for i in $DOUBLEFILES ; do
-					FILE=`ls -1 /var/log/packages |\
-						egrep -i -- "^${i}-[^-]+-(${ARCH}|noarch)-"`
+					FILE=$(ls -1 /var/log/packages |\
+						egrep -i -- "^${i}-[^-]+-(${ARCH}|fw|noarch)-")
 					FILES="$FILES $FILE"
 				done
 				showlist "$FILES" remove
@@ -677,7 +852,7 @@ Select your action (B/R): "
 			;;
 			*)
 				echo -e "\n\
-Ok! slackpkg won't do anything now. But, please, do something to fix it.\n"
+Okay - slackpkg won't do anything now, but please, do something to fix it.\n"
 				cleanup
 			;;
 		esac
